package betterwithmods.common.blocks.mechanical;

import betterwithmods.api.block.IAxle;
import betterwithmods.api.block.IMechanical;
import betterwithmods.api.block.IMechanicalBlock;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.BWSounds;
import betterwithmods.common.blocks.BlockRotate;
import betterwithmods.util.DirUtils;
import betterwithmods.util.MechanicalUtil;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.resources.I18n;
import net.minecraft.client.util.ITooltipFlag;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import javax.annotation.Nullable;
import java.util.List;
import java.util.Random;

public class BlockGearbox extends BlockRotate implements IMechanicalBlock, IMechanical {
    public static final PropertyBool ISACTIVE = PropertyBool.func_177716_a("ison");

    public BlockGearbox() {
        super(Material.field_151575_d);
        this.func_149711_c(2.0F);
        this.func_149675_a(true);
        this.func_180632_j(func_176223_P().func_177226_a(DirUtils.FACING, EnumFacing.UP).func_177226_a(ISACTIVE, false));
    }


    @Override
    public void func_190948_a(ItemStack stack, @Nullable World player, List<String> tooltip, ITooltipFlag advanced) {
        tooltip.add(I18n.func_135052_a("tooltip.gearbox.name"));
    }

    @Override
    public int func_149738_a(World world) {
        return 10;
    }

    @Override
    public IBlockState getStateForPlacement(World world, BlockPos pos, EnumFacing side, float flX, float flY, float flZ, int meta, EntityLivingBase placer, EnumHand hand) {
        IBlockState state = super.getStateForPlacement(world, pos, side, flX, flY, flZ, meta, placer, hand);
        return setFacingInBlock(state, side.func_176734_d());
    }

    @Override
    public void func_180633_a(World world, BlockPos pos, IBlockState state, EntityLivingBase entity, ItemStack stack) {
        EnumFacing facing = DirUtils.convertEntityOrientationToFacing(entity, EnumFacing.NORTH);
        world.func_175656_a(pos, world.func_180495_p(pos).func_177226_a(DirUtils.FACING, facing));
    }

    @Override
    public void nextState(World world, BlockPos pos, IBlockState state) {
        world.func_175656_a(pos, state.func_177231_a(DirUtils.FACING).func_177226_a(ISACTIVE,false));
    }


    @Override
    public void func_176213_c(World world, BlockPos pos, IBlockState state) {
        super.func_176213_c(world, pos, state);
        world.func_180497_b(pos, this, 10, 5);
    }

    @Override
    public void func_180650_b(World world, BlockPos pos, IBlockState state, Random rand) {
        boolean gettingPower = isInputtingMechPower(world, pos);
        boolean isOn = isGearboxOn(world, pos);
        boolean isRedstonePowered = world.func_175687_A(pos) > 0;

        if (isRedstonePowered)
            gettingPower = false;

        if (isOn != gettingPower) {
            if (isOn)
                setGearboxState(world, pos, false);
            else {
                setGearboxState(world, pos, true);
                world.func_184133_a(null, pos, BWSounds.WOODCREAK, SoundCategory.BLOCKS, 0.5F, world.field_73012_v.nextFloat() * 0.25F + 0.25F);
            }
        }
    }

    @Override
    public void func_189540_a(IBlockState state, World world, BlockPos pos, Block block, BlockPos other) {
        boolean isOn = isGearboxOn(world, pos);

        if (isOn) {
            boolean gettingPower = isInputtingMechPower(world, pos);
            boolean isRedstonePowered = world.func_175687_A(pos) > 0;

            if (isRedstonePowered)
                gettingPower = false;
            if (!gettingPower) {
                world.func_180497_b(pos, this, 9, 5);
                return;
            }
        }
        world.func_180497_b(pos, this, 10, 5);
    }

    public int getMechPowerProvidedToAxle(World world, BlockPos pos, EnumFacing facing) {
        if (isGearboxOn(world, pos)) {
            if (getFacing(world, pos) != facing)
                return 4;
        }
        return 0;
    }

    public EnumFacing getFacing(IBlockAccess world, BlockPos pos) {
        return getFacingFromState(world.func_180495_p(pos));
    }

    public EnumFacing getFacingFromState(IBlockState state) {
        return state.func_177229_b(DirUtils.FACING);
    }

    @Override
    public IBlockState setFacingInBlock(IBlockState state, EnumFacing facing) {
        return state.func_177226_a(DirUtils.FACING, facing);
    }

    @Override
    public boolean canRotateOnTurntable(IBlockAccess world, BlockPos pos) {
        return true;
    }

    @Override
    public boolean canRotateHorizontally(IBlockAccess world, BlockPos pos) {
        return true;
    }

    @Override
    public boolean canRotateVertically(IBlockAccess world, BlockPos pos) {
        return true;
    }

    @Override
    public void rotateAroundYAxis(World world, BlockPos pos, boolean reverse) {
        EnumFacing facing = getFacing(world, pos);
        EnumFacing newFacing = DirUtils.rotateFacingAroundY(facing, reverse);
        if (newFacing != facing) {
            if (isGearboxOn(world, pos)) {
                setGearboxState(world, pos, false);
            }

            world.func_175656_a(pos, world.func_180495_p(pos).func_177226_a(DirUtils.FACING, newFacing));

            world.func_175704_b(pos, pos);

            world.func_180497_b(pos, this, 10, 5);

            MechanicalUtil.destoryHorizontalAxles(world, pos);
        }
    }

    @Override
    public boolean canOutputMechanicalPower() {
        return true;
    }

    @Override
    public boolean canInputMechanicalPower() {
        return true;
    }

    @Override
    public boolean isInputtingMechPower(World world, BlockPos pos) {
        EnumFacing dir = getFacing(world, pos);
        return MechanicalUtil.isBlockPoweredByAxleOnSide(world, pos, dir);
    }

    @Override
    public boolean isOutputtingMechPower(World world, BlockPos pos) {
        return isGearboxOn(world, pos);
    }

    @Override
    public boolean canInputPowerToSide(IBlockAccess world, BlockPos pos, EnumFacing dir) {
        EnumFacing facing = getFacing(world, pos);
        return dir == facing;
    }

    @Override
    public void overpower(World world, BlockPos pos) {
        if (isGearboxOn(world, pos))
            breakGearbox(world, pos, world.func_180495_p(pos));
    }

    public boolean isGearboxOn(IBlockAccess world, BlockPos pos) {
        return isGearboxOnFromState(world.func_180495_p(pos));
    }

    public void setGearboxState(World world, BlockPos pos, boolean on) {
        boolean isActive = world.func_180495_p(pos).func_177229_b(ISACTIVE);
        if (on != isActive)
            world.func_175656_a(pos, world.func_180495_p(pos).func_177226_a(ISACTIVE, on));
    }

    public boolean isGearboxOnFromState(IBlockState state) {
        return state.func_177229_b(ISACTIVE);
    }

    private void emitGearboxParticles(World world, BlockPos pos, Random rand) {
        for (int i = 0; i < 5; i++) {
            float flX = pos.func_177958_n() + rand.nextFloat();
            float flY = pos.func_177956_o() + rand.nextFloat() * 0.5F + 1.0F;
            float flZ = pos.func_177952_p() + rand.nextFloat();

            world.func_175688_a(EnumParticleTypes.SMOKE_NORMAL, flX, flY, flZ, 0.0D, 0.0D, 0.0D);
        }
    }

    public void breakGearbox(World world, BlockPos pos, IBlockState state) {
        for (int i = 0; i < 10; i++) {
            world.func_184133_a(null, pos, SoundEvents.field_187927_ha, SoundCategory.BLOCKS, 1.0F, world.field_73012_v.nextFloat() * 0.1F + 0.45F);
        }
        for (int i = 0; i < 5; i++) {
            float flX = pos.func_177958_n() + world.field_73012_v.nextFloat();
            float flY = pos.func_177956_o() + world.field_73012_v.nextFloat() * 0.5F + 1.0F;
            float flZ = pos.func_177952_p() + world.field_73012_v.nextFloat();

            world.func_175688_a(EnumParticleTypes.SMOKE_LARGE, flX, flY, flZ, 0.0D, 0.0D, 0.0D);
        }
        world.func_175656_a(pos, BWMBlocks.BROKEN_GEARBOX.func_176223_P().func_177226_a(DirUtils.FACING, state.func_177229_b(DirUtils.FACING)));
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void func_180655_c(IBlockState state, World world, BlockPos pos, Random rand) {
        if (state.func_177229_b(ISACTIVE)) {
            emitGearboxParticles(world, pos, rand);
            if (rand.nextInt(50) == 0)
                world.func_184134_a(pos.func_177958_n() + 0.5D, pos.func_177956_o() + 0.5D, pos.func_177952_p() + 0.5D, BWSounds.WOODCREAK, SoundCategory.BLOCKS, 0.25F, world.field_73012_v.nextFloat() * 0.25F + 0.25F, false);
        }
    }

    @Override
    public boolean isMechanicalOn(IBlockAccess world, BlockPos pos) {
        IBlockState state = world.func_180495_p(pos);
        return isMechanicalOnFromState(state);
    }

    @Override
    public void setMechanicalOn(World world, BlockPos pos, boolean isOn) {
        world.func_175656_a(pos, world.func_180495_p(pos).func_177226_a(ISACTIVE, isOn));
    }

    @Override
    public boolean isMechanicalOnFromState(IBlockState state) {
        return state.func_177229_b(ISACTIVE);
    }

    @Override
    public int getMechPowerLevelToFacing(World world, BlockPos pos, EnumFacing dir) {
        return getMechPowerProvidedToAxle(world, pos, dir);
    }

    @Override
    public boolean isMechanicalJunction() {
        return true;
    }

    private boolean isAxle(IBlockAccess world, BlockPos origin, EnumFacing facing) {
        BlockPos pos = origin.func_177972_a(facing);
        Block block = world.func_180495_p(pos).func_177230_c();
        return block instanceof IAxle && ((IAxle) block).isAxleOrientedToFacing(world, pos, facing.func_176734_d());
    }

    @Override
    public IBlockState func_176221_a(IBlockState state, IBlockAccess world, BlockPos pos) {
        boolean[] dirs = new boolean[6];
        for (int i = 0; i < 6; i++) {
            EnumFacing facing = EnumFacing.func_82600_a(i);
            dirs[i] = isAxle(world, pos, facing) && this.getFacing(world, pos) != facing;
        }
        return state.func_177226_a(DirUtils.DOWN, dirs[0])
                .func_177226_a(DirUtils.UP, dirs[1])
                .func_177226_a(DirUtils.NORTH, dirs[2])
                .func_177226_a(DirUtils.SOUTH, dirs[3])
                .func_177226_a(DirUtils.WEST, dirs[4])
                .func_177226_a(DirUtils.EAST, dirs[5]);
    }

    @Override
    public int func_176201_c(IBlockState state) {
        int meta = isMechanicalOnFromState(state) ? 8 : 0;
        meta += state.func_177229_b(DirUtils.FACING).func_176745_a();
        return meta;
    }

    @Override
    public IBlockState func_176203_a(int meta) {
        boolean isActive = false;
        if (meta > 7) {
            isActive = true;
            meta -= 8;
        }
        return this.func_176223_P().func_177226_a(ISACTIVE, isActive).func_177226_a(DirUtils.FACING, EnumFacing.func_82600_a(meta));
    }

    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer(this, DirUtils.FACING, ISACTIVE, DirUtils.UP, DirUtils.DOWN, DirUtils.NORTH, DirUtils.SOUTH, DirUtils.WEST, DirUtils.EAST);
    }

    @Override
    public boolean func_149740_M(IBlockState state) {
        return true;
    }

    @Override
    public int func_180641_l(IBlockState blockState, World worldIn, BlockPos pos) {
        return isGearboxOn(worldIn, pos) ? 15 : 0;
    }
}
