package betterwithmods.common.blocks.mechanical;

import betterwithmods.api.block.IAxle;
import betterwithmods.api.block.IMechanical;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.blocks.BWMBlock;
import betterwithmods.util.DirUtils;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import java.util.ArrayList;
import java.util.List;

public abstract class BlockMillGenerator extends BWMBlock implements IMechanical, IAxle {
    public static final PropertyBool ISACTIVE = PropertyBool.func_177716_a("ison");
    public static final PropertyEnum<EnumFacing.Axis> AXIS = PropertyEnum.func_177709_a("axis", EnumFacing.Axis.class);
    static final AxisAlignedBB X_AABB = new AxisAlignedBB(0.0F, 0.375F, 0.375F, 1.0F, 0.625F, 0.625F);
    static final AxisAlignedBB Y_AABB = new AxisAlignedBB(0.375F, 0.0F, 0.375F, 0.625F, 1.0F, 0.625F);
    static final AxisAlignedBB Z_AABB = new AxisAlignedBB(0.375F, 0.375F, 0.0F, 0.625F, 0.625F, 1.0F);

    public BlockMillGenerator(Material material) {
        super(material);
        this.func_149672_a(SoundType.field_185848_a);
        this.func_149711_c(2.0F);
        this.func_180632_j(this.field_176227_L.func_177621_b().func_177226_a(AXIS, EnumFacing.Axis.Z).func_177226_a(ISACTIVE, false));
    }

    @Override
    public int getPowerLevel(IBlockAccess world, BlockPos pos) {
        return getPowerLevelFromState(world.func_180495_p(pos));
    }

    public int getPowerLevelFromState(IBlockState state) {
        return state.func_177229_b(ISACTIVE) ? 4 : 0;
    }

    @Override
    public int getMechPowerLevelToFacing(World world, BlockPos pos,
                                         EnumFacing dir) {
        if (world.func_180495_p(pos).func_177229_b(ISACTIVE)) {
            EnumFacing.Axis axis = world.func_180495_p(pos).func_177229_b(AXIS);
            if (dir.func_176740_k() == axis) return 4;
        }
        return 0;
    }

    @Override
    public boolean isAxleOrientedToFacing(IBlockAccess world, BlockPos pos, EnumFacing dir) {
        EnumFacing.Axis axis = world.func_180495_p(pos).func_177229_b(AXIS);
        return dir.func_176740_k() == axis;
    }

    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer(this, AXIS, ISACTIVE);
    }


    @Override
    public int func_176201_c(IBlockState state) {
        int meta = DirUtils.getLegacyAxis(state.func_177229_b(AXIS));
        if (state.func_177229_b(ISACTIVE))
            meta += 8;
        return meta;
    }

    @Override
    public IBlockState func_176203_a(int meta) {
        boolean active = meta > 7;
        if (active)
            meta -= 8;
        return this.getAxisState(DirUtils.getAxisFromLegacy(meta)).func_177226_a(ISACTIVE, active);
    }

    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @Override
    public boolean func_149686_d(IBlockState state) {
        return false;
    }

    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos) {
        switch (state.func_177229_b(AXIS)) {
            case X:
                return X_AABB;
            case Y:
                return Y_AABB;
            case Z:
            default:
                return Z_AABB;
        }
    }

    @Override
    public int func_149738_a(World world) {
        return 20;
    }

    public abstract ItemStack getGenStack(IBlockState state);

    @Override
    public List<ItemStack> getDrops(IBlockAccess world, BlockPos pos, IBlockState state, int fortune) {
        List<ItemStack> drops = new ArrayList<>();
        drops.add(new ItemStack(BWMBlocks.AXLE));
        drops.add(getGenStack(state));
        return drops;
    }

    public IBlockState getAxisState(EnumFacing.Axis axis) {
        return this.func_176223_P().func_177226_a(AXIS, axis);
    }

    public EnumFacing getAxleDirectionFromState(IBlockState state) {
        EnumFacing.Axis axis = state.func_177229_b(AXIS);
        switch (axis) {
            case Y:
                return EnumFacing.UP;
            case Z:
                return EnumFacing.SOUTH;
            case X:
            default:
                return EnumFacing.EAST;
        }
    }

    public EnumFacing getAxleDirection(World world, BlockPos pos) {
        EnumFacing.Axis axis = world.func_180495_p(pos).func_177229_b(AXIS);
        switch (axis) {
            case Y:
                return EnumFacing.UP;
            case Z:
                return EnumFacing.SOUTH;
            case X:
            default:
                return EnumFacing.EAST;
        }
    }

    @Override
    public int getAxisAlignment(IBlockAccess world, BlockPos pos) {
        IBlockState state = world.func_180495_p(pos);
        if (state.func_177230_c() == this)
            return DirUtils.getLegacyAxis(state.func_177229_b(AXIS));
        return DirUtils.getLegacyAxis(EnumFacing.Axis.Y);
    }

    @Override
    public boolean isMechanicalJunction() {
        return true;
    }
}
