package betterwithmods.common.blocks.tile;

import betterwithmods.api.block.IMechanical;
import betterwithmods.api.block.IMechanicalBlock;
import betterwithmods.api.capabilities.MechanicalCapability;
import betterwithmods.api.tile.IMechanicalPower;
import betterwithmods.common.blocks.mechanical.BlockAdvGearbox;
import betterwithmods.util.MechanicalUtil;
import net.minecraft.block.state.IBlockState;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.capabilities.Capability;

import javax.annotation.Nonnull;

public class TileEntityGearbox extends TileEntity implements ITickable, IMechanicalPower {
    private int powerLevel;
    private byte outputs;
    private int refreshTime = 0;

    @Override
    public int getMinimumInput(EnumFacing facing) {
        return 1;
    }

    @Override
    public void func_73660_a() {
        if (func_145838_q() instanceof BlockAdvGearbox) {
            if (((BlockAdvGearbox) func_145838_q()).isGearboxOn(func_145831_w(), field_174879_c)) {
                if (refreshTime == 20) {
                    findOutputs();
                } else {
                    refreshTime++;
                }
            } else if (powerLevel != 0)
                powerLevel = 0;
        }
    }

    private void findOutputs() {
        outputs = 0;
        for (EnumFacing facing : EnumFacing.field_82609_l) {
            if (((BlockAdvGearbox) func_145838_q()).canInputPowerToSide(func_145831_w(), field_174879_c, facing)) {
                if (powerLevel != getMechanicalInput(facing))
                    powerLevel = getMechanicalInput(facing);
            } else if (MechanicalUtil.isAxle(func_145831_w().func_180495_p(field_174879_c.func_177972_a(facing)).func_177230_c())) {
                outputs++;
            }
        }
    }

    @Override
    public boolean hasCapability(@Nonnull Capability<?> capability, @Nonnull EnumFacing facing) {
        return capability == MechanicalCapability.MECHANICAL_POWER
                || super.hasCapability(capability, facing);
    }

    @Nonnull
    @Override
    public <T> T getCapability(@Nonnull Capability<T> capability, @Nonnull EnumFacing facing) {
        if (capability == MechanicalCapability.MECHANICAL_POWER)
            return MechanicalCapability.MECHANICAL_POWER.cast(this);
        return super.getCapability(capability, facing);
    }

    @Override
    public int getMechanicalOutput(EnumFacing facing) {
        if (func_145838_q() instanceof IMechanical) {
            if (((IMechanical) func_145838_q()).getMechPowerLevelToFacing(func_145831_w(), field_174879_c, facing) > 0) {
                if (outputs > 0)
                    return powerLevel / outputs;
                else
                    return powerLevel;
            }
        }
        return 0;
    }

    @Override
    public int getMechanicalInput(EnumFacing facing) {
        int power = powerLevel;
        if (func_145838_q() instanceof IMechanicalBlock) {
            if (((IMechanicalBlock) func_145838_q()).canInputPowerToSide(func_145831_w(), func_174877_v(), facing)) {
                power = Math.min(MechanicalUtil.searchForAdvMechanical(func_145831_w(), func_174877_v(), facing), getMaximumInput(facing));
            }
        }
        return power;
    }

    @Override
    public int getMaximumInput(EnumFacing facing) {
        return 100;
    }

    @Override
    public void func_145839_a(NBTTagCompound tag) {
        super.func_145839_a(tag);
        powerLevel = tag.func_74762_e("Power");
        outputs = tag.func_74771_c("Outputs");
    }

    @Nonnull
    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound tag) {
        NBTTagCompound t = super.func_189515_b(tag);
        tag.func_74768_a("Power", powerLevel);
        tag.func_74774_a("Outputs", outputs);
        return t;
    }

    @Override
    public boolean shouldRefresh(World world, BlockPos pos, @Nonnull IBlockState oldState, @Nonnull IBlockState newState) {
        return oldState.func_177230_c() != newState.func_177230_c();
    }
}
